# -*- coding: utf8 -*-

import sys
import unittest

from draw import Draw
import graph

class TestProcessPlayers(unittest.TestCase):
    """Test the players file processing."""

    def test_empty(self):
        """No players."""
        r = process_players("")
        self.assertEqual(r, [])

    def test_no_separation(self):
        """All lines need to have two ':'."""
        self.assertRaises(ValueError, process_players, "foo:foo:bar\no:sep 55")
        self.assertRaises(ValueError, process_players, "nnosep 55")

    def test_bad_current_points(self):
        """Points needs to be numeric."""
        self.assertRaises(ValueError, process_players, "foo:bar:66")

    def test_bad_losing_points(self):
        """Points needs to be numeric."""
        self.assertRaises(ValueError, process_players, "foo:33:alpha")

    def test_comment(self):
        """Comments are ignored."""
        r = process_players("#foo\na:5:2")
        self.assertEqual(r, [(u"a", 5, 2)])

    def test_empty_line(self):
        """One player."""
        r = process_players("\n\nK :3: 3\n")
        self.assertEqual(r, [(u"K", 3, 3)])

    def test_one(self):
        """One player."""
        r = process_players("John Doe : \t55: 22 \n")
        self.assertEqual(r, [(u"John Doe", 55, 22)])

    def test_two(self):
        """Two players."""
        r = process_players("John Doe : \t55 :22\nFoo:  12 :1")
        self.assertEqual(r, [(u"John Doe", 55, 22), (u"Foo", 12, 1)])

    def test_unicode(self):
        """Support utf8 files."""
        r = process_players(u"Miño: 77 :1".encode('utf8'))
        self.assertEqual(r, [(u"Miño", 77, 1)])


class TestProcessTournament(unittest.TestCase):
    """Test the tournament file processing."""

    def test_comment(self):
        """Comments are ignored."""
        r = process_tournament("#foo")
        self.assertEqual(r, ([], [], []))

    def test_empty_line(self):
        """One player."""
        r = process_tournament("\n\n\n")
        self.assertEqual(r, ([], [], []))

    def test_bad_signature(self):
        """First letter must be p or r."""
        t = "01: Foo Bar\n p 02: Barro\n r 120: 01 02\n r 400:"
        self.assertRaises(ValueError, process_tournament, t)

        t = "p 01: Foo Bar\n k 02: Barro\n r 120: 01 02\n r 400:"
        self.assertRaises(ValueError, process_tournament, t)

        t = "p 01: Foo Bar\n p 02: Barro\n 120: 01 02\n r 400:"
        self.assertRaises(ValueError, process_tournament, t)

        t = "p 01: Foo Bar\n p 02: Barro\n r 120: 01 02\n l 400:"
        self.assertRaises(ValueError, process_tournament, t)

    def test_no_separation(self):
        """First letter must be p or r."""
        t = "01 Foo Bar\n p 02: Barro\n r 120: 01 02\n r 400:"
        self.assertRaises(ValueError, process_tournament, t)

        t = "01: Foo Bar\n p 02: Barro\n r 120 01 02\n r 400:"
        self.assertRaises(ValueError, process_tournament, t)

    def test_all_ok_empty_2(self):
        """All data ok."""
        t = "p 01: Foo Bar\n p 02: Barro\n r 120: 01 02\n r 400:"
        draw_base, points, draw_played = process_tournament(t)
        self.assertEqual(draw_base, ["Foo Bar", "Barro"])
        self.assertEqual(points, [120, 400])
        self.assertEqual(draw_played, [])

    def test_all_ok_empty_4(self):
        """All data ok."""
        t = "p 01: Foo\n p 02: A B\n p 03: bar\n p 04: zoofy\n"\
                "r  20 : 01 02 03 04\n r 100: \nr 500:"
        draw_base, points, draw_played = process_tournament(t)
        self.assertEqual(draw_base, ["Foo", "A B", "bar", "zoofy"])
        self.assertEqual(points, [20, 100, 500])
        self.assertEqual(draw_played, [])

    def test_all_ok_notempty_4(self):
        """All data ok."""
        t = "p 01: Foo\n p 02: A B\n p 03: bar\n p 04: zoofy\n"\
            "r  20 : 01 02 03 04\n r 100: 03\nr 500:"
        draw_base, points, draw_played = process_tournament(t)
        self.assertEqual(draw_base, ["Foo", "A B", "bar", "zoofy"])
        self.assertEqual(points, [20, 100, 500])
        self.assertEqual(draw_played, [["bar", None]])


def process_players(data):
    """Process the players file."""
    result = []
    for line in data.split("\n"):
        line = line.strip()
        if not line or line[0] == '#':
            continue

        player, points, losing = line.split(":")
        player = player.strip().decode('utf8')
        points = int(points)
        losing = int(losing)
        result.append((player, points, losing))

    return result


def process_tournament(data):
    """Process the tournament file."""
    players = {}
    tourn = []
    points = []
    for line in data.split("\n"):
        line = line.strip()
        if not line or line[0] == '#':
            continue

        base, rest = line.split(":")
        sign, cod = base.split()
        cod = int(cod)
        rest = rest.strip()
        if sign not in ('r', 'p'):
            raise ValueError("Bad signature: %r" % sign)

        if sign == 'p':
            players[cod] = rest
        else:
            points.append(cod)
            if rest:
                tourn.append([players[int(n)] for n in rest.split()])

    if tourn:
        base = tourn[0]
        draw = tourn[1:]
    else:
        base = []
        draw = []

    # fill the remaining draws
    filled = []
    for i, row in enumerate(draw):
        lenrow = len(base) / (2 ** (i + 1))
        filled.append((row + ([None]*lenrow))[:lenrow])
    return base, points, filled


def main(f_players, f_tournament):
    with open(f_players) as fh:
        players = process_players(fh.read())

    with open(f_tournament) as fh:
        draw_base, points, draw_played = process_tournament(fh.read())

    # prepare previous results
    previous = [(x, y-z) for x, y, z in players]

    # prepare draw and calculate
    d = Draw(draw_base, points, previous)
    d.fill_draw(draw_played)
    res = d.calculate()

    # show the ranking
    to_graph = []
    for i, (player, _, _) in enumerate(players):
#        print
#        print player, res[player].keys()
        possible = res[player].keys()
        it = (i+1, player, min(possible), max(possible))
        print it
        to_graph.append(it)

    # graph!
    graph.positions(to_graph)


if __name__ == '__main__':
    if len(sys.argv) != 3:
        print "Usar tennis.py <players> <tournament>"
        sys.exit()

    main(*sys.argv[1:3])
